'use strict';

var bridgesCoreSdk = require('@rsksmart/bridges-core-sdk');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol, Iterator */


function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

function initBridge(bridgeName, config) {
    return __awaiter(this, void 0, void 0, function* () {
        switch (bridgeName) {
            case 'flyover': {
                const flyoverPkg = yield Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespaceDefault(require('@rsksmart/flyover-sdk')); });
                return new flyoverPkg.Flyover(config);
            }
            case 'tokenbridge': {
                const tokenbridgePkg = yield Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespaceDefault(require('@rsksmart/tokenbridge-sdk')); });
                return new tokenbridgePkg.TokenBridge(config);
            }
            default:
                throw new Error('unexisting bridge');
        }
    });
}
function getConfig(initConfig, bridge) {
    const particularConfig = initConfig.bridges[bridge];
    if (particularConfig == null) {
        throw new Error('unexisting bridge');
    }
    const config = Object.assign(Object.assign({}, particularConfig), { network: initConfig.network, allowInsecureConnections: initConfig.allowInsecureConnections, rskConnection: initConfig.rskConnection });
    return config;
}
function init(initParams) {
    return __awaiter(this, void 0, void 0, function* () {
        let bridge;
        const bridges = {};
        const bridgesNames = Object.keys(initParams.bridges);
        for (const bridgeName of bridgesNames) {
            try {
                bridge = yield initBridge(bridgeName, getConfig(initParams, bridgeName));
                bridges[bridgeName] = bridge;
            }
            catch (e) {
                throw convertToBridgeError(bridgeName, e);
            }
        }
        return bridges;
    });
}
function convertToBridgeError(bridgeName, e) {
    if ((e === null || e === void 0 ? void 0 : e.code) !== 'ERR_MODULE_NOT_FOUND') {
        throw e;
    }
    const packages = {
        flyover: '@rsksmart/flyover-sdk',
        tokenbridge: '@rsksmart/tokenbridge-sdk'
    };
    return new bridgesCoreSdk.BridgeError({
        timestamp: Date.now(),
        recoverable: false,
        message: 'Missing bridge package',
        details: `You're trying to use a bridge which is not in your package, please install ${packages[bridgeName]} to start using ${bridgeName}`,
        product: 'core'
    });
}

exports.init = init;
